#!/bin/bash
# ------------------------------------------------------------------------------
# Copyright 2023 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.

# Copyright 2022 - 2023 Open Text.
#
#  The only warranties for products and services of Open Text and its affiliates and licensors (“Open Text”)
#  are as may be set forth in the express warranty statements accompanying such products and services.
#  Nothing herein should be construed as constituting an additional warranty. Open Text shall not be liable
#  for technical or editorial errors or omissions contained herein. The information contained herein is
#  subject to change without notice.
# ------------------------------------------------------------------------------
#
# script to check all umc config health

UWHITE="\033[4;37m"
URED="\033[4;31m"
UGREEN="\033[4;32m"
GREEN="\033[1;32m"
RED="\033[0;31m"
BLUE="\033[0;94m"
YELLOW="\033[1;33m"
NC="\033[0m"

container_name="edirapi-container"
UMC_SERVICE_CONFIGURED=$(grep SERVICE_CONFIGURED /etc/sysconfig/novell/umc | cut -d '=' -f2 | cut -d "\"" -f2)
LOGFILE="/var/opt/novell/log/umc/apps/umc-server/health.log"
startTime=$(date +%s.%N)
autoFix=false
onlyDbCheck=false
declare -i serviceCheckIssue=0
declare -i serviceCheckAutoFix=0
declare -i edirapiCheckIssue=0
declare -i keyCheckIssue=0
declare -i secObjIssue=0
declare -i dbTableIssue=0
declare -i dbWarnings=0
declare -i dbTableAutoFix=0
declare -i nodeModuleIssue=0
declare -i nodeModuleAutoFix=0
VERSION_NUMBER="1.0.0" #script version

logFile_check() {
	if [[ -f $LOGFILE ]]; then
		echo "#=================================================#" >>$LOGFILE
		echo "" >>$LOGFILE
		printf "#=================================================#\n\n"
		echo "          [Start UMC SERVER HEALTH CHECK]               " >>$LOGFILE
		printf "          [Start UMC SERVER HEALTH CHECK]               \n"
		echo >>$LOGFILE
		printf "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z)]\n\n"
		echo "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z)]" >>$LOGFILE
		echo >>$LOGFILE
		echo "   Autofix value is set to $autoFix" >>$LOGFILE
		echo >>$LOGFILE
	else
		touch $LOGFILE
		echo "#=================================================#" >>$LOGFILE
		echo "" >>$LOGFILE
		printf "#=================================================#\n\n"
		echo "          [Start UMC SERVER HEALTH CHECK]               " >>$LOGFILE
		printf "          [Start UMC SERVER HEALTH CHECK]               \n"
		echo >>$LOGFILE
		printf "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z)]\n\n"
		echo "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z)]" >>$LOGFILE
		echo >>$LOGFILE
		echo "   Autofix value is set to $autoFix" >>$LOGFILE
		echo >>$LOGFILE
	fi
}

service_check() {

	printf "#=================================================#\n\n"
	printf "             [Service Status Check]                \n\n"
	echo "#=================================================#" >>$LOGFILE
	echo >>$LOGFILE
	echo "             [Service Status Check]                " >>$LOGFILE

	array=('apache2' 'postgresql' 'ndsd' 'microfocus-umc-server' 'microfocus-umc-backend' 'docker' 'docker-edirapi')

	#filter all services to different array on basis of the status of service
	for i in ${array[@]}; do
		result=$(systemctl is-active ${i}.service)
		if [ "$result" = "active" ]; then
			active_service+=" "${i}
		elif [ "$result" = "inactive" ]; then
			inactive_service+=" "${i}
		elif [ "$result" = "failed" ]; then
			failed_service+=" "${i}
		fi
	done

	#check for active service and dispaly
	for i in ${active_service[@]}; do
		result=$(systemctl is-active ${i}.service)
		printf "${i}.service is ${UGREEN}${result}${NC}\n"
		echo "${i}.service is ${result}" >>$LOGFILE
	done

	#check for failed service and dispaly
	for i in ${failed_service[@]}; do
		result=$(systemctl is-active ${i}.service)
		printf "${i}.service is ${URED}${result}${NC}\n"
		serviceCheckIssue+=1
		echo "${i}.service is ${result}" >>$LOGFILE
		if [[ "$autoFix" = true ]]; then
			printf "${RED}ERROR: ${i}${NC} is in failed state. Starting the service.....\n\n"
			echo "ERROR: ${i} is in failed state. Starting the service....." >>$LOGFILE
			$(systemctl restart ${i}.service)
			printf "Exit value for restarting ${i}.service is : $?\n\n"
			echo "Exit value for restarting ${i}.service is : $?" >>$LOGFILE
			if [ $? == 0 ]; then
				serviceCheckAutoFix+=1
			fi
		else
			printf "${RED}ERROR: ${i}${NC} is in failed state. Make sure service is up and running by executing ${BLUE}systemctl restart ${i}.service${NC}\n\n"
			echo "ERROR: ${i} is in failed state. Make sure service is up and running by executing systemctl restart ${i}.service" >>$LOGFILE
		fi
	done

	#check for inactive service and dispaly
	for i in ${inactive_service[@]}; do
		result=$(systemctl is-active ${i}.service)
		printf "${i}.service is ${URED}${result}${NC}\n"
		serviceCheckIssue+=1
		echo "${i}.service is ${result}" >>$LOGFILE
		if [[ "$autoFix" = true ]]; then
			printf "${RED}ERROR: ${i}${NC} is in inactive state. Starting the service.....\n\n"
			echo "ERROR: ${i} is in inactive state. Starting the service....." >>$LOGFILE
			$(systemctl restart ${i}.service)
			printf "Exit value for restarting ${i}.service is : $?\n\n"
			echo "Exit value for restarting ${i}.service is : $?" >>$LOGFILE
			if [ $? == 0 ]; then
				serviceCheckAutoFix+=1
			fi
		else
			printf "${RED}ERROR: ${i}${NC} is in inactive state. Make sure service is up and running by executing ${BLUE}systemctl restart ${i}.service${NC}\n\n"
			echo "ERROR: ${i} is in inactive state. Make sure service is up and running by executing systemctl restart ${i}.service" >>$LOGFILE
		fi
	done

	printf "\n#=================================================#\n"
	echo "#=================================================#" >>$LOGFILE
}

edirapi_check() {

	if [[ "$UMC_SERVICE_CONFIGURED" == "yes" ]]; then
		printf "#=================================================#\n\n"
		echo "#=================================================#" >>$LOGFILE
		echo >>$LOGFILE
		printf "                  [Docker Details]                 \n\n"
		echo "                  [Docker Details]                 " >>$LOGFILE

		# Check docker running status
		if [ $(docker inspect -f '{{.State.Running}}' $container_name) = "true" ]; then
			printf "edirapi-container is ${GREEN}running${NC} and started at $(docker inspect -f '{{ .State.StartedAt }}' $container_name)\n\n"
			echo "edirapi-container is running and started at $(docker inspect -f '{{ .State.StartedAt }}' $container_name)" >>$LOGFILE
		else
			edirapiCheckIssue+=1
			printf "${RED}ERROR: edirapi-container is not running${NC}\n\n"
			echo "ERROR: edirapi-container is not running" >>$LOGFILE
		fi

		# check for edirapi port
		edirApi_port=$(grep "^[[:blank:]]*EDIR_REST_API_PORT=" /opt/novell/umc/apps/umc-server/prod.env | sed s/^[[:blank:]]*EDIR_REST_API_PORT=//)
		edirApi_host=$(grep "^[[:blank:]]*EDIR_REST_API_SERVER=" /opt/novell/umc/apps/umc-server/prod.env | sed s/^[[:blank:]]*EDIR_REST_API_SERVER=//)
		if [ -n "$edirApi_port" ]; then
			edirApiPortCheck=$(nc -zv ${edirApi_host} ${edirApi_port})
			if [ $? == 0 ]; then
				printf "edirapi port ${edirApi_host}:${edirApi_port} is opened in firewall\n\n"
				echo "edirapi port ${edirApi_host}:${edirApi_port} is opened in firewall" >>$LOGFILE
			else
				edirapiCheckIssue+=1
				printf "${RED}ERROR: edirapi port ${edirApi_host}:${edirApi_port} is not opened in firewall${NC}\nOpen the port in firewall by running ${BLUE}firewall-cmd --zone=public --permanent --add-port=$edirApi_port/tcp${NC} on host ${edirApi_host} and then reload the firewall by running ${BLUE}systemctl restart firewalld.service${NC}\n\n"
				echo "ERROR: edirapi port ${edirApi_host}:${edirApi_port} is not opened in firewall. Open the port in firewall by running 'firewall-cmd --zone=public --permanent --add-port=$edirApi_port/tcp' on host ${edirApi_host} and then reload the firewall by running 'systemctl restart firewalld.service'" >>$LOGFILE
			fi
		fi

		printf "\n#=================================================#\n"
		echo "#=================================================#" >>$LOGFILE
	fi
}

apacheModule_check() {

	# check the apache module needed for umc
	if [[ "$UMC_SERVICE_CONFIGURED" == "yes" ]]; then
		printf "#=================================================#\n\n"
		echo "#=================================================#" >>$LOGFILE
		echo >>$LOGFILE
		printf "               [Apache Module Check]              \n\n"
		echo "               [Apache Module Check]              " >>$LOGFILE

		header_mod=$(apachectl -M 2>/dev/null | grep headers_module)
		if [ -z "$header_mod" ]; then
			serviceCheckIssue+=1
			if [[ "$autoFix" = true ]]; then
				printf "${RED}ERROR: apache headers module is disabled${NC}\nEnabling the apache headers module.....\n\n"
				echo "ERROR: apache headers module is disabled. Enabling the apache headers module....." >>$LOGFILE
				$(a2enmod headers)
				printf "Exit value for enabling apache headers module is : $?\n\n"
				echo "Exit value for enabling apache headers module is : $?" >>$LOGFILE
				if [ $? == 0 ]; then
					serviceCheckAutoFix+=1
				fi
				$(systemctl restart apache2.service)
				printf "Exit value for restarting apache2.service is : $?\n\n"
				echo "Exit value for restarting apache2.service is : $?" >>$LOGFILE
			else
				printf "${RED}ERROR: apache headers module is disabled${NC}\nEnable it by running ${BLUE}a2enmod headers${NC} and restart the ${GREEN}apache2${NC} by executing ${BLUE}systemctl restart apache2.service${NC}\n\n"
				echo "ERROR: apache headers module is disabled. Enable it by running 'a2enmod headers' and restart the apache2 by executing 'systemctl restart apache2.service'" >>$LOGFILE
			fi
		else
			printf "apache headers module is ${GREEN}enabled${NC}\n\n"
			echo "apache headers module is enabled" >>$LOGFILE
		fi

		header_mod=$(apachectl -M 2>/dev/null | grep proxy_http_module)
		if [ -z "$header_mod" ]; then
			serviceCheckIssue+=1
			if [[ "$autoFix" = true ]]; then
				printf "${RED}ERROR: apache proxy_http module is disabled${NC}\nEnabling the apache proxy_http module.....\n\n"
				echo "ERROR: apache proxy_http module is disabled. Enabling the apache proxy_http module....." >>$LOGFILE
				$(a2enmod proxy_http)
				if [ $? == 0 ]; then
					serviceCheckAutoFix+=1
				fi
				printf "Exit value for enabling proxy_http module is : $?\n\n"
				echo "Exit value for enabling proxy_http module is : $?" >>$LOGFILE
				$(systemctl restart apache2.service)
				printf "Exit value for restarting apache2.service is : $?\n\n"
				echo "Exit value for restarting apache2.service is : $?" >>$LOGFILE
			else
				printf "${RED}ERROR: apache proxy_http module is disabled${NC}\nEnable it by running ${BLUE}a2enmod proxy_http${NC} and restart the ${GREEN}apache2${NC} by executing ${BLUE}systemctl restart apache2.service${NC}\n\n"
				echo "ERROR: apache proxy_http module is disabled. Enable it by running 'a2enmod proxy_http' and restart the apache2 by executing 'systemctl restart apache2.service'" >>$LOGFILE
			fi
		else
			printf "apache proxy_http module is ${GREEN}enabled${NC}\n\n"
			echo "apache proxy_http module is enabled" >>$LOGFILE
		fi

		printf "\n#=================================================#\n"
		echo "#=================================================#" >>$LOGFILE
	fi
}

key_check() {

	# Certificate validity
	printf "#=================================================#\n\n"
	echo "#=================================================#" >>$LOGFILE
	echo >>$LOGFILE
	printf "               [Certificate Check]              \n\n"
	echo "               [Certificate Check]              " >>$LOGFILE

	publicKey=$(grep "^[[:blank:]]*PUBLIC_KEY_PATH=" /opt/novell/umc/apps/umc-server/prod.env | sed s/^[[:blank:]]*PUBLIC_KEY_PATH=//)
	publicKeyExp=$(openssl x509 -enddate -noout -in ${publicKey} | cut -d "=" -f2)
	printf "Certificate will expire after ${publicKeyExp}\n\n"
	echo "Certificate will expire after ${publicKeyExp}" >>$LOGFILE

	sanDetail=$(openssl x509 -noout -ext subjectAltName -in ${publicKey})
	echo "${sanDetail}" >>$LOGFILE
	printf "${sanDetail}\n"

	#Private RSA Key Check
	privateKey=$(grep "^[[:blank:]]*PRIVATE_KEY_PATH=" /opt/novell/umc/apps/umc-server/prod.env | sed s/^[[:blank:]]*PRIVATE_KEY_PATH=//)
	privateKeyCheck=$(openssl rsa -in ${privateKey} -check -noout)

	printf "\n${privateKeyCheck}\n"
	echo "${privateKeyCheck}" >>$LOGFILE

	printf "\n#=================================================#\n"
	echo "#=================================================#" >>$LOGFILE

}

secObj_check() {

	printf "#=================================================#\n\n"
	echo "#=================================================#" >>$LOGFILE
	echo >>$LOGFILE
	printf "         [Edirectory Security Object Check]         \n\n"
	echo "         [Edirectory Security Object Check]         " >>$LOGFILE

	securityObject=$(/usr/bin/ldapsearch -x -b cn=umcConfig,cn=Security -s base)
	if [ $? == 0 ]; then
		printf "\n${GREEN}$(/usr/bin/ldapsearch -x -b cn=umcConfig,cn=Security -s base | grep dn:)${NC} is present.\n"
		echo "$(/usr/bin/ldapsearch -x -b cn=umcConfig,cn=Security -s base | grep dn:) is present." >>$LOGFILE
		printf "${GREEN}$(/usr/bin/ldapsearch -o ldif-wrap=no -x -b cn=umcConfig,cn=Security -s base | grep umcJwkUrl)${NC}\n\n"
		echo "Value of $(/usr/bin/ldapsearch -o ldif-wrap=no -x -b cn=umcConfig,cn=Security -s base | grep umcJwkUrl)" >>$LOGFILE
	else
		secObjIssue+=1
		printf "\n${RED}ERROR: umcConfig Object is missing${NC}. It might be of below reason \n1. UMC is not configured in tree\n2. UMC server is not configured properly\n"
		echo "ERROR: umcConfig Object is missing. It might be of below reason 1. UMC is not configured in tree 2. UMC server is not configured properly" >>$LOGFILE
	fi

	printf "\n#=================================================#\n"
	echo "#=================================================#" >>$LOGFILE

}

getDbUrl() {

	dbUrl=$(grep "^[[:blank:]]*DB_URL=" /opt/novell/umc/apps/umc-server/prod.env | sed s/^[[:blank:]]*DB_URL=//)

	# extract the protocol
	proto="$(echo $dbUrl | grep '://' | sed -e's,^\(.*://\).*,\1,g')"
	# remove the protocol
	url=$(echo $dbUrl | sed -e s,$proto,,g)

	# extract the user and password (if any)
	userpass="$(echo $url | grep @ | cut -d@ -f1)"

	# extract the host -- updated
	hostport=$(echo $url | sed -e s,$userpass@,,g | cut -d/ -f1)
	port=$(echo $hostport | grep : | cut -d: -f2)
	if [ -n "$port" ]; then
		host=$(echo $hostport | grep : | cut -d: -f1)
	else
		host=$hostport
	fi

	# extract the db name of umc
	path="$(echo $url | grep / | cut -d/ -f2-)"
}

dbTable_check() {

	if [[ "$UMC_SERVICE_CONFIGURED" == "yes" ]]; then
		getDbUrl
		# Check DB Configuration
		printf "#=================================================#\n\n"
		echo "#=================================================#" >>$LOGFILE
		echo >>$LOGFILE
		printf "                     [DB Check]                   \n\n"
		echo "                     [DB Check]                    " >>$LOGFILE

		if [[ "$path" = 'postgres' ]] || [[ "$path" = 'umc' ]]; then
			printf "Warning : DB is not migrated to use umc specific DB. You can run below command to migrate db.....\n"
			printf "Execute command -> ${BLUE}/opt/novell/umc/scripts/migrate_table.py${NC}\n\n"
			echo "Warning : Still using ${path} as DB. DB is not migrated to use umc specific DB." >>$LOGFILE
			dbWarnings+=1
		fi

		portCheck=$(nc -zv ${host} ${port})
		if [ $? == 0 ]; then
			if $(/opt/novell/umc/scripts/execute_db_command.py check_db_exist); then
				printf "DB ${path} is ${GREEN}present and accessible${NC}\n\n"
				echo "DB ${path} is present and accessible" >>$LOGFILE
			else
				dbTableIssue+=1
				if [[ "$autoFix" = true ]]; then
					printf "${RED}ERROR: DB \"${path}\" is not present in postgres${NC}. Creating the missing DB.....\n"
					echo "ERROR: DB ${path} is not present in postgres. Creating the missing DB....." >>$LOGFILE
					$(/opt/novell/umc/scripts/execute_db_command.py create_umc_db)
					printf "Exit value for ${path} DB creation is : $?\n\n"
					echo "Exit value for ${path} DB creation is : $?" >>$LOGFILE
					if [ $? == 0 ]; then
						dbTableAutoFix+=1
					fi
				else
					if [[ "$onlyDbCheck" = true ]]; then
						printf "${RED}ERROR: DB \"${path}\" is not present in postgres${NC}. Creating the missing DB :\n"
						echo "ERROR: DB ${path} is not present in postgres. Creating the missing DB :" >>$LOGFILE
						read -p "Enter \"yes\" if you want to create the ${path} DB:" user_choice
						if [[ ${user_choice,,} == "y" || ${user_choice,,} == "yes" ]]; then
							$(/opt/novell/umc/scripts/execute_db_command.py create_umc_db)
							printf "Exit value for ${path} DB creation is : $?\n\n"
							echo "Exit value for ${path} DB creation is : $?" >>$LOGFILE
							if [ $? == 0 ]; then
								dbTableAutoFix+=1
							fi
						else
							printf "DB ${path} is not created\n\n"
							echo "DB ${path} is not created" >>$LOGFILE
						fi
					elif [[ "$onlyDbCheck" = false ]]; then
						printf "${RED}ERROR: DB \"${path}\" is not present in postgres${NC}.\nCreate the missing DB by running ${BLUE}/sbin/umcServiceHealth.sh -dautofix${NC}\n\n"
						echo "ERROR: DB ${path} is not present in postgres.Create the missing DB by running /sbin/umcServiceHealth.sh -dautofix" >>$LOGFILE
					fi
				fi
			fi

			if ! $(/opt/novell/umc/scripts/execute_db_command.py check_table_exist --table-name umc_login_sessions); then
				dbTableIssue+=1
				if [[ "$autoFix" = true ]]; then
					printf "${RED}ERROR: Table \"umc_login_sessions\" is not present in DB${NC}. Creating the missing table.....\n"
					echo "ERROR: Table umc_login_sessions is not present in DB. Creating the missing table....." >>$LOGFILE
					$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name umc_login_sessions >/dev/null)
					printf "Exit value for umc_login_sessions table creation is : $?\n\n"
					echo "Exit value for umc_login_sessions table creation is : $?" >>$LOGFILE
					if [ $? == 0 ]; then
						dbTableAutoFix+=1
					fi
				else
					if [[ "$onlyDbCheck" = true ]]; then
						printf "${RED}ERROR: Table \"umc_login_sessions\" is not present in DB${NC}. Create the missing table : \n"
						echo "ERROR: Table umc_login_sessions is not present in DB. Create the missing table by running:" >>$LOGFILE
						read -p "Enter \"yes\" if you want to create the umc_login_sessions table:" user_choice
						if [[ ${user_choice,,} == "y" || ${user_choice,,} == "yes" ]]; then
							$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name umc_login_sessions >/dev/null)
							printf "Exit value for umc_login_sessions table creation is : $?\n\n"
							echo "Exit value for umc_login_sessions table creation is : $?" >>$LOGFILE
							if [ $? == 0 ]; then
								dbTableAutoFix+=1
							fi
						else
							printf "Table umc_login_sessions is not created\n\n"
							echo "Table umc_login_sessions is not created" >>$LOGFILE
						fi
					elif [[ "$onlyDbCheck" = false ]]; then
						printf "${RED}ERROR: Table \"umc_login_sessions\" is not present in DB${NC}.\nCreate the missing table by running ${BLUE}/sbin/umcServiceHealth.sh -dautofix${NC}\n\n"
						echo "ERROR: Table umc_login_sessions is not present in DB. Create the missing table by running /sbin/umcServiceHealth.sh -dautofix" >>$LOGFILE
					fi
				fi
			else
				printf "Table umc_login_sessions is ${GREEN}present and accessible${NC}\n\n"
				echo "Table umc_login_sessions is present and accessible" >>$LOGFILE
			fi

			if ! $(/opt/novell/umc/scripts/execute_db_command.py check_table_exist --table-name action); then
				dbTableIssue+=1
				if [[ "$autoFix" = true ]]; then
					printf "${RED}ERROR: Table \"action\" is not present in DB${NC}. Creating the missing table.....\n"
					echo "ERROR: Table action is not present in DB. Creating the missing table." >>$LOGFILE
					$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name action >/dev/null)
					printf "Exit value for action table creation is : $?\n\n"
					echo "Exit value for action table creation is : $?" >>$LOGFILE
					if [ $? == 0 ]; then
						dbTableAutoFix+=1
					fi
				else
					if [[ "$onlyDbCheck" = true ]]; then
						printf "${RED}ERROR: Table \"action\" is not present in DB${NC}. Create the missing table : \n"
						echo "ERROR: Table action is not present in DB. Create the missing table :" >>$LOGFILE
						read -p "Enter \"yes\" if you want to create the action table:" user_choice
						if [[ ${user_choice,,} == "y" || ${user_choice,,} == "yes" ]]; then
							$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name action >/dev/null)
							printf "Exit value for action table creation is : $?\n\n"
							echo "Exit value for action table creation is : $?" >>$LOGFILE
							if [ $? == 0 ]; then
								dbTableAutoFix+=1
							fi
						else
							printf "Table action is not created\n\n"
							echo "Table action is not created" >>$LOGFILE
						fi
					elif [[ "$onlyDbCheck" = false ]]; then
						printf "${RED}ERROR: Table \"action\" is not present in DB${NC}.\nCreate the missing table by running ${BLUE}/sbin/umcServiceHealth.sh -dautofix${NC}\n\n"
						echo "ERROR: Table action is not present in DB. Create the missing table by running /sbin/umcServiceHealth.sh -dautofix" >>$LOGFILE
					fi
				fi
			else
				printf "Table action is ${GREEN}present and accessible${NC}\n\n"
				echo "Table action is present and accessible" >>$LOGFILE
			fi

			if ! $(/opt/novell/umc/scripts/execute_db_command.py check_table_exist --table-name tree); then
				dbTableIssue+=1
				if [[ "$autoFix" = true ]]; then
					printf "${RED}ERROR: Table \"tree\" is not present in DB${NC}. Creating the missing table.....\n"
					echo "ERROR: Table tree is not present in DB. Creating the missing table." >>$LOGFILE
					$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name tree >/dev/null)
					printf "Exit value for tree table creation is : $?\n\n"
					echo "Exit value for tree table creation is : $?" >>$LOGFILE
					if [ $? == 0 ]; then
						dbTableAutoFix+=1
					fi
				else
					if [[ "$onlyDbCheck" = true ]]; then
						printf "${RED}ERROR: Table \"tree\" is not present in DB${NC}. Create the missing table : \n"
						echo "ERROR: Table tree is not present in DB. Create the missing table :" >>$LOGFILE
						read -p "Enter \"yes\" if you want to create the tree table:" user_choice
						if [[ ${user_choice,,} == "y" || ${user_choice,,} == "yes" ]]; then
							$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name tree >/dev/null)
							printf "Exit value for tree table creation is : $?\n\n"
							echo "Exit value for tree table creation is : $?" >>$LOGFILE
							if [ $? == 0 ]; then
								dbTableAutoFix+=1
							fi
						else
							printf "Table tree is not created\n\n"
							echo "Table tree is not created" >>$LOGFILE
						fi
					elif [[ "$onlyDbCheck" = false ]]; then
						printf "${RED}ERROR: Table \"tree\" is not present in DB${NC}.\nCreate the missing table by running ${BLUE}/sbin/umcServiceHealth.sh -dautofix${NC}\n\n"
						echo "ERROR: Table tree is not present in DB. Create the missing table by running /sbin/umcServiceHealth.sh -dautofix" >>$LOGFILE
					fi
				fi
			else
				printf "Table tree is ${GREEN}present and accessible${NC}\n\n"
				echo "Table tree is present and accessible" >>$LOGFILE
			fi

			if ! $(/opt/novell/umc/scripts/execute_db_command.py check_table_exist --table-name user_profile); then
				dbTableIssue+=1
				if [[ "$autoFix" = true ]]; then
					printf "${RED}ERROR: Table \"user_profile\" is not present in DB${NC}. Creating the missing table.....\n"
					echo "ERROR: Table user_profile is not present in DB. Creating the missing table." >>$LOGFILE
					$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name user_profile >/dev/null)
					printf "Exit value for user_profile table creation is : $?\n\n"
					echo "Exit value for user_profile table creation is : $?" >>$LOGFILE
					if [ $? == 0 ]; then
						dbTableAutoFix+=1
					fi
				else
					if [[ "$onlyDbCheck" = true ]]; then
						printf "${RED}ERROR: Table \"user_profile\" is not present in DB${NC}. Create the missing table : \n"
						echo "ERROR: Table user_profile is not present in DB. Create the missing table :" >>$LOGFILE
						read -p "Enter \"yes\" if you want to create the user_profile table:" user_choice
						if [[ ${user_choice,,} == "y" || ${user_choice,,} == "yes" ]]; then
							$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name user_profile >/dev/null)
							printf "Exit value for user_profile table creation is : $?\n\n"
							echo "Exit value for user_profile table creation is : $?" >>$LOGFILE
							if [ $? == 0 ]; then
								dbTableAutoFix+=1
							fi
						else
							printf "Table user_profile is not created\n\n"
							echo "Table user_profile is not created" >>$LOGFILE
						fi
					elif [[ "$onlyDbCheck" = false ]]; then
						printf "${RED}ERROR: Table \"user_profile\" is not present in DB${NC}.\nCreate the missing table by running ${BLUE}/sbin/umcServiceHealth.sh -dautofix${NC}\n\n"
						echo "ERROR: Table user_profile is not present in DB. Create the missing table by running /sbin/umcServiceHealth.sh -dautofix" >>$LOGFILE
					fi
				fi
			else
				printf "Table user_profile is ${GREEN}present and accessible${NC}\n\n"
				echo "Table user_profile is present and accessible" >>$LOGFILE
			fi

			if ! $(/opt/novell/umc/scripts/execute_db_command.py check_table_exist --table-name report); then
				dbTableIssue+=1
				if [[ "$autoFix" = true ]]; then
					printf "${RED}ERROR: Table \"report\" is not present in DB${NC}. Creating the missing table.....\n"
					echo "ERROR: Table report is not present in DB. Creating the missing table." >>$LOGFILE
					$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name report >/dev/null)
					printf "Exit value for report table creation is : $?\n\n"
					echo "Exit value for report table creation is : $?" >>$LOGFILE
					if [ $? == 0 ]; then
						dbTableAutoFix+=1
					fi
				else
					if [[ "$onlyDbCheck" = true ]]; then
						printf "${RED}ERROR: Table \"report\" is not present in DB${NC}. Create the missing table : \n"
						echo "ERROR: Table report is not present in DB. Create the missing table :" >>$LOGFILE
						read -p "Enter \"yes\" if you want to create the report table:" user_choice
						if [[ ${user_choice,,} == "y" || ${user_choice,,} == "yes" ]]; then
							$(/opt/novell/umc/scripts/execute_db_command.py create_table --table-name report >/dev/null)
							printf "Exit value for report table creation is : $?\n\n"
							echo "Exit value for report table creation is : $?" >>$LOGFILE
							if [ $? == 0 ]; then
								dbTableAutoFix+=1
							fi
						else
							printf "Table report is not created\n\n"
							echo "Table report is not created" >>$LOGFILE
						fi
					elif [[ "$onlyDbCheck" = false ]]; then
						printf "${RED}ERROR: Table \"report\" is not present in DB${NC}.\nCreate the missing table by running ${BLUE}/sbin/umcServiceHealth.sh -dautofix${NC}\n\n"
						echo "ERROR: Table report is not present in DB. Create the missing table by running /sbin/umcServiceHealth.sh -dautofix" >>$LOGFILE
					fi
				fi
			else
				printf "Table report is ${GREEN}present and accessible${NC}\n\n"
				echo "Table report is present and accessible" >>$LOGFILE
			fi

		else
			printf "${RED}ERROR: Check database port ${port} is opened in firewall and ${host} is reachable${NC}\n\n"
			echo "ERROR: Check database port is opened in firewall and is reachable" >>$LOGFILE
		fi

		printf "\n#=================================================#\n"
		echo "#=================================================#" >>$LOGFILE
	fi
}

nodeModule_check() {

	if [[ "$UMC_SERVICE_CONFIGURED" == "yes" ]]; then
		printf "#=================================================#\n\n"
		echo "#=================================================#" >>$LOGFILE
		echo >>$LOGFILE
		printf "                   [Node Module Check]                   \n\n"
		echo "                   [Node Module Check]                    " >>$LOGFILE

		if [ -d "/opt/novell/umc/apps/node_modules/" ]; then
			printf "Node Modules directory is ${GREEN}present${NC}\n\n"
			echo "Node Modules directory is present" >>$LOGFILE
		else
			nodeModuleIssue+=1
			if [[ "$autoFix" = true ]] && [[ -f "/opt/novell/umc/apps/node_modules.tar.xz" ]]; then
				printf "${RED}ERROR: Node Modules directory is missing${NC}. Extracting the node_modules....\n\n"
				echo "ERROR: Node Modules directory is missing. Extracting the node_modules...." >>$LOGFILE
				$(tar Jxf /opt/novell/umc/apps/node_modules.tar.xz -C /opt/novell/umc/apps/)
				printf "Exit value for node_modules extraxtion is : $?\n\n"
				echo "Exit value for node_modules extraxtion is : $?" >>$LOGFILE
				if [ $? == 0 ]; then
					nodeModuleAutoFix+=1
				fi
				$(systemctl restart microfocus-umc-server.service)
			elif [[ "$autoFix" = false ]] && [[ -f "/opt/novell/umc/apps/node_modules.tar.xz" ]]; then
				printf "${RED}ERROR: Node Modules directory is missing${NC}.\n\n"
				printf "Extract the node_modules by following steps -\n1. tar Jxf /opt/novell/umc/apps/node_modules.tar.xz -C /opt/novell/umc/apps/ \n2. systemctl restart microfocus-umc-server.service\n\n"
				echo "ERROR: Node Modules directory is missing." >>$LOGFILE
				echo "Extract the node_modules by following steps - 1. tar Jxf /opt/novell/umc/apps/node_modules.tar.xz -C /opt/novell/umc/apps/ 2. systemctl restart microfocus-umc-server.service" >>$LOGFILE
			elif [[ ! -f "/opt/novell/umc/apps/node_modules.tar.xz" ]]; then
				printf "${RED}ERROR: Node Modules directory and tar file is missing${NC}.\n\n"
				printf "Make Sure \"microfocus-oes-umc-node_modules\" package is installed.\nOnce /opt/novell/umc/apps/node_modules.tar.xz file is present.\n"
				printf "Extract the node_modules by following steps -\n1. tar Jxf /opt/novell/umc/apps/node_modules.tar.xz -C /opt/novell/umc/apps/ \n2. systemctl restart microfocus-umc-server.service\n\n"
				echo "ERROR: Node Modules directory and tar file is missing." >>$LOGFILE
				echo "Make Sure microfocus-oes-umc-node_modules package is installed. Once /opt/novell/umc/apps/node_modules.tar.xz file is present." >>$LOGFILE
				echo "Extract the node_modules by following steps - 1. tar Jxf /opt/novell/umc/apps/node_modules.tar.xz -C /opt/novell/umc/apps/ 2. systemctl restart microfocus-umc-server.service" >>$LOGFILE
			fi
		fi
		printf "#=================================================#\n"
		echo "#=================================================#" >>$LOGFILE
	fi
}

finalSummary() {

	printf "#=================================================#\n\n"
	echo "#=================================================#" >>$LOGFILE
	echo >>$LOGFILE
	echo "                    [Summary]                       " >>$LOGFILE
	printf "                    [Summary]                       \n"
	if [ $serviceCheckIssue -gt 0 ]; then
		echo "      Service Check $serviceCheckIssue Issue Found" >>$LOGFILE
		printf "      ${RED}Service Check $serviceCheckIssue Issue Found${NC}\n"
	fi

	if [ $serviceCheckAutoFix -gt 0 ]; then
		echo "      Service Check $serviceCheckAutoFix Issue AutoFixed" >>$LOGFILE
		printf "      ${GREEN}Service Check $serviceCheckAutoFix Issue AutoFixed${NC}\n"
	fi

	if [ $edirapiCheckIssue -gt 0 ]; then
		echo "      EdirApi Check $edirapiCheckIssue Issue Found" >>$LOGFILE
		printf "\n      ${RED}EdirApi Check $edirapiCheckIssue Issue Found${NC}\n"
	fi

	if [ $keyCheckIssue -gt 0 ]; then
		echo "      Certificate Check $keyCheckIssue Issue Found" >>$LOGFILE
		printf "\n      ${RED}Certificate Check $keyCheckIssue Issue Found${NC}\n"
	fi

	if [ $secObjIssue -gt 0 ]; then
		echo "      UMC Object Check $secObjIssue Issue Found" >>$LOGFILE
		printf "\n      ${RED}UMC Object Check $secObjIssue Issue Found${NC}\n"
	fi

	if [ $dbTableIssue -gt 0 ]; then
		echo "      DB Check $dbTableIssue Issue Found" >>$LOGFILE
		printf "\n      ${RED}DB Check $dbTableIssue Issue Found${NC}\n"
	fi

	if [ $dbWarnings -gt 0 ]; then
		echo "      DB Check $dbWarnings Warning Found" >>$LOGFILE
		printf "\n      ${RED}DB Check $dbWarnings Warning Found${NC}\n"
	fi

	if [ $dbTableAutoFix -gt 0 ]; then
		echo "      DB Check $dbTableAutoFix Issue AutoFixed" >>$LOGFILE
		printf "      ${GREEN}DB Check $dbTableAutoFix Issue AutoFixed${NC}\n"
	fi

	if [ $nodeModuleIssue -gt 0 ]; then
		echo "      Node Module Check $nodeModuleIssue Issue Found" >>$LOGFILE
		printf "\n      ${RED}Node Module Check $nodeModuleIssue Issue Found${NC}\n"
	fi

	if [ $nodeModuleAutoFix -gt 0 ]; then
		echo "      Node Module Check $nodeModuleAutoFix Issue AutoFixed" >>$LOGFILE
		printf "      ${GREEN}Node Module Check $nodeModuleAutoFix Issue AutoFixed${NC}\n"
	fi

	if [ $dbTableIssue -eq 0 ] && [ $dbWarnings -eq 0 ] && [ $secObjIssue -eq 0 ] && [ $keyCheckIssue -eq 0 ] && [ $edirapiCheckIssue -eq 0 ] && [ $serviceCheckIssue -eq 0 ] && [ $nodeModuleIssue -eq 0 ]; then
		echo "                 No Issue Found                   " >>$LOGFILE
		printf "                 No Issue Found                   \n"
	fi

	echo "#=================================================#" >>$LOGFILE
	printf "\n#=================================================#\n"
}

indicateEnd() {

	echo "#=================================================#" >>$LOGFILE
	printf "#=================================================#\n\n"
	duration=$(awk "BEGIN {print $(date +%s.%N) - $startTime}")
	execution_time=$(printf "%.2f seconds" $duration)
	echo "       END on : [$(date +%Y-%m-%d:%H:%M:%S:%Z)]                      " >>$LOGFILE
	printf "      END on : [$(date +%Y-%m-%d:%H:%M:%S:%Z)]                      \n"
	printf "      Script Execution Time: $execution_time\n\n"
	echo "       Script Execution Time: $execution_time" >>$LOGFILE
	echo "#=================================================#" >>$LOGFILE
	printf "#=================================================#\n"
	exit 0
}

runAllCheck() {

	logFile_check
	# Check Service Status related to UMC
	service_check
	# Check edir api status
	edirapi_check
	# Check apache module is enable or disabled
	apacheModule_check
	# Check the validity of private and public key
	key_check
	# Check edirectory security object
	secObj_check
	# Check Database reachable or not and Table is created or not
	dbTable_check
	# Check Node Modules is properly extracted or not
	nodeModule_check
	finalSummary
	indicateEnd
}

Usage() {
	echo ""
	echo "umcServiceHealth Script Version ${VERSION_NUMBER} checks health of all the services and provide necessary action in case of failure"
	echo ""
	echo "Usage: umcServiceHealth.sh [options]"
	echo "          -h|--help                  Prints this summary"
	echo "          -s|--service-check         Check health of dependent services"
	echo "          -e|--edirapi-check         Check health of edirApi"
	echo "          -c|--cert-check            Check health of certificate"
	echo "          -u|--edirObj-check         Check health of umcObject"
	echo "          -d|--db-check              Check health of database"
	echo "          -n|--nodeModule-check      Check Node Modules"
	echo "          -a|--all-check             Run all health check for UMC"
	echo ""
	echo "Log Location - ${LOGFILE}"
	echo ""
	echo "Autofix is available for --service-check, --db-check, --nodeModule-check and --all-check option."
	echo "Autofix will automatically fix the issue detected."
	echo ""
	echo "Example -> sh umcServiceHealth.sh -dautofix OR sh umcServiceHealth.sh --db-check=autofix"
	echo "Example -> sh umcServiceHealth.sh -aautofix OR sh umcServiceHealth.sh --all-check=autofix"
	echo ""
	echo "If you dont want to enable auto fix then run script without passing 'autofix'"
	echo "It will populate the solution if any issue is detected."
	echo ""
	echo "Example -> sh umcServiceHealth.sh -d OR sh umcServiceHealth.sh --db-check"
	exit 1
}

if test $# -eq 0; then
	Usage
fi

#Read the options
TEMP=$(getopt -o s::ecud::n::a::h --long service-check::,edirapi-check,cert-check,edirObj-check,db-check::,nodeModule-check::,all-check::,help -- "$@")
eval set -- "$TEMP"

#Extract options and their arguments into variables.
while true; do
	case "$1" in
	-s | --service-check)
		case "$2" in
		"autofix")
			autoFix=true
			logFile_check
			service_check
			apacheModule_check
			finalSummary
			indicateEnd
			exit
			;;
		*)
			autofix=false
			logFile_check
			service_check
			apacheModule_check
			finalSummary
			indicateEnd
			exit
			;;
		esac
		;;
	-e | --edirapi-check)
		logFile_check
		edirapi_check
		finalSummary
		indicateEnd
		exit
		;;
	-c | --cert-check)
		logFile_check
		key_check
		finalSummary
		indicateEnd
		exit
		;;
	-u | --edirObj-check)
		logFile_check
		secObj_check
		finalSummary
		indicateEnd
		exit
		;;
	-d | --db-check)
		case "$2" in
		"autofix")
			autoFix=true
			logFile_check
			dbTable_check
			finalSummary
			indicateEnd
			exit
			;;
		*)
			autofix=false
			onlyDbCheck=true
			logFile_check
			dbTable_check
			finalSummary
			indicateEnd
			exit
			;;
		esac
		;;
	-n | --nodeModule-check)
		case "$2" in
		"autofix")
			autoFix=true
			logFile_check
			nodeModule_check
			finalSummary
			indicateEnd
			exit
			;;
		*)
			autofix=false
			logFile_check
			nodeModule_check
			finalSummary
			indicateEnd
			exit
			;;
		esac
		;;
	-a | --all-check)
		case "$2" in
		"autofix")
			autoFix=true
			runAllCheck
			exit
			;;
		*)
			autofix=false
			runAllCheck
			exit
			;;
		esac
		;;
	-h | --help)
		Usage
		;;
	\?)
		# Invalid option
		echo "Error: Invalid option"
		Usage
		;;
	*)
		Usage
		;;
	esac
done
