-- table to maintain session data for login

CREATE TABLE IF NOT EXISTS "umc_login_sessions" () WITH (OIDS=FALSE);

ALTER TABLE "umc_login_sessions" ADD COLUMN IF NOT EXISTS "sid" varchar NOT NULL COLLATE "default";

ALTER TABLE "umc_login_sessions" ADD COLUMN IF NOT EXISTS "sess" json NOT NULL;

ALTER TABLE "umc_login_sessions" ADD COLUMN IF NOT EXISTS "expire" timestamp(6) NOT NULL;

ALTER TABLE "umc_login_sessions" ADD COLUMN IF NOT EXISTS "user_id" varchar;

ALTER TABLE "umc_login_sessions" ADD CONSTRAINT "session_pkey" PRIMARY KEY ("sid") NOT DEFERRABLE INITIALLY IMMEDIATE;

CREATE INDEX IF NOT EXISTS "IDX_session_expire" ON "umc_login_sessions" ("expire");

-- table to log user actions

CREATE TABLE IF NOT EXISTS "action" () WITH (OIDS=FALSE);

ALTER TABLE "action" ADD COLUMN IF NOT EXISTS "id" uuid NOT NULL DEFAULT gen_random_uuid ();

ALTER TABLE "action" ADD COLUMN IF NOT EXISTS "timestamp" timestamp(6) NOT NULL;

ALTER TABLE "action" ADD COLUMN IF NOT EXISTS "user" varchar NOT NULL COLLATE "default";

ALTER TABLE "action" ADD COLUMN IF NOT EXISTS "api" varchar NOT NULL;

ALTER TABLE "action" ADD COLUMN IF NOT EXISTS "operation" varchar NOT NULL;

ALTER TABLE "action" ADD COLUMN IF NOT EXISTS "params" json NOT NULL;

ALTER TABLE "action" ADD COLUMN IF NOT EXISTS "status" varchar NOT NULL COLLATE "default";

ALTER TABLE "action" ADD CONSTRAINT "id_pkey" PRIMARY KEY ("id") NOT DEFERRABLE INITIALLY IMMEDIATE;

CREATE INDEX IF NOT EXISTS "IDX_user" ON "action" ("user");

-- a lookup table for edir tree-name resolution

CREATE TABLE IF NOT EXISTS "tree" () WITH (OIDS=FALSE);

ALTER TABLE "tree" ADD COLUMN IF NOT EXISTS "name" varchar NOT NULL;

ALTER TABLE "tree" ADD COLUMN IF NOT EXISTS "network_interfaces" varchar[] NOT NULL;

ALTER TABLE "tree" ADD CONSTRAINT "name_pkey" PRIMARY KEY ("name") NOT DEFERRABLE INITIALLY IMMEDIATE;

-- table to store the per-user profile and settings information

CREATE TABLE IF NOT EXISTS "user_profile" () WITH (OIDS=FALSE);

ALTER TABLE "user_profile" ADD COLUMN IF NOT EXISTS "user_id" varchar;

ALTER TABLE "user_profile" ADD COLUMN IF NOT EXISTS "key" varchar NOT NULL;

ALTER TABLE "user_profile" ADD COLUMN IF NOT EXISTS "value" json NOT NULL;

ALTER TABLE "user_profile" ADD COLUMN IF NOT EXISTS "update_at" timestamp NOT NULL;

ALTER TABLE "user_profile" ADD CONSTRAINT "user_and_key" UNIQUE("user_id","key");

-- table to store the report response information

CREATE TABLE IF NOT EXISTS "report" () WITH (OIDS=FALSE);

DO $$ BEGIN
    CREATE TYPE report_type AS ENUM ('cluster');
EXCEPTION
    WHEN duplicate_object THEN null;
END $$;
-- add new types using the below command
-- ALTER TYPE report_type ADD VALUE IF NOT EXISTS 'new_type1' AFTER 'cluster';
-- ALTER TYPE report_type ADD VALUE IF NOT EXISTS 'new_type2' AFTER 'new_type1'; -- each new type should be added AFTER the last type

DO $$ BEGIN
    CREATE TYPE report_status AS ENUM ('in_progress','available', 'failed');
EXCEPTION
    WHEN duplicate_object THEN null;
END $$;

ALTER TABLE "report" ADD COLUMN IF NOT EXISTS "id" uuid NOT NULL DEFAULT gen_random_uuid ();

ALTER TABLE "report" ADD COLUMN IF NOT EXISTS "type" report_type NOT NULL;

ALTER TABLE "report" ADD COLUMN IF NOT EXISTS "status" report_status NOT NULL DEFAULT 'in_progress';

ALTER TABLE "report" ADD COLUMN IF NOT EXISTS "entity_name" varchar NOT NULL;

ALTER TABLE "report" ADD COLUMN IF NOT EXISTS "response" json DEFAULT NULL;

ALTER TABLE "report" ADD COLUMN IF NOT EXISTS "requested_at" timestamp NOT NULL DEFAULT NOW();

ALTER TABLE "report" ADD COLUMN IF NOT EXISTS "created_at" timestamp DEFAULT NULL;

ALTER TABLE "report" ADD CONSTRAINT "report_id"  PRIMARY KEY ("id") NOT DEFERRABLE INITIALLY IMMEDIATE;